@@ -2,7 +2,8 @@
     if (interactive()) {
         packageStartupMessage('data.table ',as.character(packageVersion("data.table")),'  For help type: help("data.table")')
     }
-    "Please read FAQ 2.23 (vignette('datatable-faq')) which explains (at some length) why data.table adds one line to base::cbind.data.frame and base::rbind.data.frame. If there is a better solution we will gladly change it."
+    "Please read FAQ 2.23 (vignette('datatable-faq')) which explains in detail why data.table adds one line to base::cbind.data.frame and base::rbind.data.frame. If there is a better solution we will gladly change it."
+    # Commented as a character string so this message is retained and seen by anyone who types data.table:::.onAttach
     tt = base::cbind.data.frame
     ss = body(tt)
     if (class(ss)!="{") ss = as.call(c(as.name("{"), ss))
@@ -10,7 +11,7 @@
         ss = ss[c(1,NA,2:length(ss))]
         ss[[2]] = parse(text="if (inherits(..1,'data.table')) return(data.table(...,key=key(..1)))")[[1]]
         body(tt)=ss
-        unlockBinding("cbind.data.frame",baseenv())
+        (unlockBinding)("cbind.data.frame",baseenv())
         assign("cbind.data.frame",tt,envir=asNamespace("base"),inherits=FALSE)
         lockBinding("cbind.data.frame",baseenv())
     }
@@ -21,12 +22,12 @@
         ss = ss[c(1,NA,2:length(ss))]
         ss[[2]] = parse(text="if (inherits(..1,'data.table')) return(`.rbind.data.table`(...))")[[1]]
         body(tt)=ss
-        unlockBinding("rbind.data.frame",baseenv())
+        (unlockBinding)("rbind.data.frame",baseenv())
         assign("rbind.data.frame",tt,envir=asNamespace("base"),inherits=FALSE)
         lockBinding("rbind.data.frame",baseenv())
     }
-    "Please read FAQ 2.23 (vignette('datatable-faq')) which explains (at some length) why data.table adds one line to base::cbind.data.frame and base::rbind.data.frame. If there is a better solution we will gladly change it."
-    # Commented as a character string so it's retained and seen by anyone who types data.table:::.onAttach
+    "Please read FAQ 2.23 (vignette('datatable-faq')) which explains in detail why data.table adds one line to base::cbind.data.frame and base::rbind.data.frame. If there is a better solution we will gladly change it."
+    # Commented as a character string so this message is retained and seen by anyone who types data.table:::.onAttach
     invisible()
 }
 
